/* XMMS UADE plugin
 *
 * Copyright (C) 2000-2001  Heikki Orsila
 *                          heikki.orsila@ee.tut.fi
 *                          http://www.ee.tut.fi/~heikki
 *
 * This plugin is based on xmms 0.9.6 wavplayer input plugin code. Since
 * then all code has been rewritten.
 *
 * Configure and About based on code of the null-plugin by Hvard Kvlen.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/time.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <xmms/plugin.h>
#include <xmms/configfile.h>
#include <xmms/util.h>

#include "deli.h"
#include "gui.h"
#include "btree.h"
#include "checksum.h"

/* Mutex */
static pthread_mutex_t uade_mutex = PTHREAD_MUTEX_INITIALIZER;
#define uade_lock() pthread_mutex_lock(&uade_mutex)
#define uade_unlock() pthread_mutex_unlock(&uade_mutex)

/* FUNCTION DECLARATIONS */

static void deli_init(void);
static int is_our_file(char *filename);
static void play_file(char *filename);
static void stop(void);
static void deli_pause(short p);
static int get_time(void);
static void get_song_info(char *filename, char **title, int *length);
static void clean_up(void);

static int signal_slave(int signum);

static int check_my_file(char*,char*,char*);
static void waitforuaetoreact(void);
static int check_if_file_exists(char*,char*);
static void my_fileinfo(char*);

static void db_add_song_data(tnode **root, char *md5sum, int length);
static int db_check_song(tnode *root, const char *filename);
static int db_get_content_hash(char *hash, const char *filename);
static int db_get_name_hash(char *hash, const char *filename);
static int db_read_data(tnode **root, const char *filename);
static void db_write_data(const char *filename, tnode *root);
static void setup_databases(void);
static int uadedbwriteroutine(tnode *node, void *arg);


/* GLOBAL VARIABLE DECLARATIONS */

InputPlugin deli_ip = {
	0,
	0,
	"UADE " VERSION,
	deli_init,        /* input plugin initialization (run uade) */
	deli_about,       /* about gui */
	deli_configure,   /* configuration gui */
	is_our_file,      /* checks if a file is for uade playback */
	0,
	play_file,        /* plays a song */
	stop,             /* stops playing */
	deli_pause,       /* pauses playback */
	deli_seeksubsong, /* changes subsong */
	0,
	get_time,         /* sets xmms playtime */
	0,
	0,
	clean_up,         /* this deletes posix tmp-file */
	0,
	0,
	0,
	0,
	get_song_info,    /* gives xmms general info about played song */
	my_fileinfo,      /* file_info_box */
	0
};

char *deli_file = 0;      /* name of played song (zero if nothing is played) */

int uadeformats_is_cached = 0;

/* LOCAL VARIABLE DECLARATIONS */

static pid_t slavepid = 0;       /* pid of uade executable */

/* timeout value in seconds for songs that don't stop ever or play otherwise
   too long (timeout<=0 => unlimited play */
static int timeout = -1;

/* some stupid and/or bad state variables */
static int playingbit = 0;
static pthread_t decode_thread;
static int uade_frequency = 44100;
static int uade_nchannels = 2;
static int uade_bytespersample = 2;

static uade_msgstruct *uade_struct = 0;  /* shared memory pointer */
static char mapfilename[256];            /* name of shared mem file */
static char uadename[256];               /* name of uade executable */
static char uadeshellparameter[256];     /* a shell parameter passed
					    -config=CONFIGFILE */
static char current_song_content_hash[33];
static char current_song_name_hash[33];
static tnode *songcontenttree = 0;      /* md5 content hash keys */
static tnode *songnametree = 0;         /* full name keys (including path) */
static char uade_content_db_filename[256];
static char uade_name_db_filename[256];
static int db_loaded = 0;

/* this variable is used for recording playtimes to the uade database */
static int cumulative_playtime;
static int database_playtime;

static int uade_is_operational = 0;      /* >0 if initialization is success */

struct uade_format_list {
  struct uade_format_list *next;
  char ext[16];
  char player[64];
  char playerfilepath[256];
};

static struct uade_format_list *uadeformats_head = 0;

static int uade_bigendian = 0;
static AFormat uade_format = FMT_S16_LE;


InputPlugin *get_iplugin_info(void) {
  return &deli_ip;
}


static void deli_init(void) {
  char uadeconfigname[256];      /* uade configuration file name */
  const int length = 65536;      /* length of shared mem file */
  FILE* sharedmemFILE;
  int fd,i;
  short betest;
  char *beptr = (char*) &betest;

  goto lbC000400;
  fprintf(stderr,"Contrary to popular belief, GOTOs are nice\n");
 lbC000400:

  /* test if cpu is in big-endian mode */
  betest = 0x1234;
  uade_bigendian = (beptr[0] == 0x34) ? 0:1;
  uade_format = (uade_bigendian) ? FMT_S16_BE : FMT_S16_LE;

  /* read configuration in configure.c module */
  deli_configread();

  /* create temporary name for mmap'ed shared memory file */
  mapfilename[0] = 0;
  (void) tmpnam(mapfilename);

  /* open and create blank file for shared memory file with size of "length" */
  if(!(sharedmemFILE = fopen(mapfilename,"w+"))) {
    fprintf(stderr,"uade: can not create mmap file (fopen)\n");
    return;
  }
  for(i=0;i<length;i+=256) fwrite(uadename,1,256,sharedmemFILE);
  fclose(sharedmemFILE);
  fd = open(mapfilename,O_RDWR);
  if(fd < 0) {
    fprintf(stderr,"uade: can not open mmap file (open)\n");
    return;
  }
  uade_struct = (uade_msgstruct*) mmap(0,length,PROT_READ|PROT_WRITE,MAP_SHARED,fd,0);
  if(uade_struct == MAP_FAILED) {
    fprintf(stderr,"uade: can not mmap shared mem file\n");
    return;
  }

  /* initialize uade_msgstruct to zeros in shared memory */
  memset(uade_struct,0,sizeof(uade_msgstruct));

  /* give xmms plugin's pid to uade (it watches if the plugin dies) */
  uade_struct->masterpid = getpid();

  /* safe initialization of certain variables */
  uadename[0] = 0;
  uadeconfigname[0] = 0;
  uade_struct->scorename[0] = 0;

  /* expands names to uade data dir */
  resolve_path(uadename,uade_path);
  resolve_path(uade_struct->scorename,score_path);
  resolve_path(uadeconfigname,uade_conf_path);

  /* Set NTSC/PAL mode for uade */
  set_ntsc_pal(use_ntsc);
  
  // MAKE SURE uade EXECUTABLE EXISTS ACCORDING TO DELICONFIG FILE
  if(check_if_file_exists(uadename,"UADE EXECUTABLE")==0) {
    return;
  }
  // MAKE SURE SOUNDCOREFILE EXISTS ACCORDING TO DELICONFIG FILE
  if(check_if_file_exists(uade_struct->scorename,"SOUNDCOREFILE")==0) {
    return;
  }

  /* set uadeshellparameter switches (makes uade read the right uaerc file) */
  strcpy(uadeshellparameter,"-config=");
  strcat(uadeshellparameter,uadeconfigname);

  /* mark that initialization seems to be a success */
  uade_is_operational = 1;
}

static int signal_slave(int signum) {
  if(slavepid>0)
    return kill(slavepid,signum);
  else
    return -1;
}

static void forkexec(void) {
  fprintf(stderr,"uade: fork & exec uade executable\n");
  /* fork the task to execute uade in the slave */
  slavepid = fork();
  /* forked slave will call the uade executable */
  if(slavepid == 0) {
    char* newargv[]={uadename,uadeshellparameter,"-slave",mapfilename,0};
    execv(uadename,newargv);
    fprintf(stderr,"UADE EXECUTABLE exiting (this is a bad thing)\n");
    fprintf(stderr,"If I were you, I wouldn't try to play songs that need UADE (it could freeze your X)\n");
    abort();
  }
}


static void setup_databases(void) {
  char globaldbname[256];
  int ch,cg,nh,ng;

  if(db_loaded) return;

  fprintf(stderr,"uade: loading databases\n");

  uade_content_db_filename[0] = 0;
  strcpy(uade_content_db_filename, getenv("HOME"));
  strcpy(uade_name_db_filename, uade_content_db_filename);
  strcat(uade_content_db_filename, "/.xmms/uade/db-content");
  strcat(uade_name_db_filename, "/.xmms/uade/db-name");
  ch = db_read_data(&songcontenttree, uade_content_db_filename);
  strcpy(globaldbname, UADEDATADIR);
  strcat(globaldbname, "/db-content");
  cg = db_read_data(&songcontenttree, globaldbname);
  if(ch == 0 && cg == 0) {
    fprintf(stderr,"uade: couldn't read song content database: Following files were tried:\n");
    fprintf(stderr,"\t1. %s\n",uade_content_db_filename);
    fprintf(stderr,"\t2. %s\n",globaldbname);
  }
  nh = db_read_data(&songnametree, uade_name_db_filename);
  strcpy(globaldbname, UADEDATADIR);
  strcat(globaldbname, "/db-name");
  ng = db_read_data(&songnametree, globaldbname);
  /* important */
  db_loaded = 1;
}


void resolve_path(char *dst, char *src) {
  if(src[0]=='/') {
    strcpy(dst,src);
  } else if (src[0]=='~') {
    strcpy(dst,getenv("HOME"));
    src++;
    strcat(dst,src);
  } else {
    FILE *test;
    strcpy(dst,getenv("HOME"));
    strcat(dst,"/.xmms/uade/");
    strcat(dst,src);
    if((test=fopen(dst,"r"))) {
      fclose(test);
      return;
    }
    strcpy(dst,UADEDATADIR);
    strcat(dst,"/");
    strcat(dst,src);
    if((test=fopen(dst,"r"))) {
      fclose(test);
    } else {
      dst[0]=0;
    }
  }
}


static int check_if_file_exists(char *filename,char *id) {
  if(filename[0]=='\0') {
    fprintf(stderr,"XMMS DELIPLUGIN ERROR:\n");
    fprintf(stderr,"%s name was not found from $(HOME)/.xmms/uade/deliconfig file!\n",id);
    return 0;
  } else {
    FILE* file;
    file=fopen(filename,"r");
    if(file==0) {
      fprintf(stderr,"XMMS DELIPLUGIN ERROR:\n");
      fprintf(stderr,"%s '%s' was not found (it must exist!) !\n",id,filename);
      fprintf(stderr,"You might be able to solve this problem by\n");
      fprintf(stderr,"\t1. Creating the necessary files (follow UADE installation procedures (make userinstall)\n");
      fprintf(stderr,"\t\tor\n");
      fprintf(stderr,"\t2. Editing ~/.xmms/config (section deli)");
      return 0;
    } else {
      fclose(file);
    }
  }
  return 1;
}


static void clean_up(void) {
  if (mapfilename) {
    fprintf(stderr,"Removing tempfile %s (used for shared mem)\n",mapfilename);
    (void) remove(mapfilename);
  }
  db_write_data(uade_content_db_filename, songcontenttree);
  db_write_data(uade_name_db_filename, songnametree);
}


static int is_our_file(char *filename) {
  if( uade_is_operational == 0) return FALSE;
  return check_my_file(filename,0,0);
}


static int check_my_file(char* filename,char* format,char* playername) {
  FILE* formatsfile;
  char a[256],b[256];
  int x;
  int rval=FALSE;
  char pre[256],post[256];
  char *c,*filename2;
  char formatsfilename[256];

  // set default timeout (unlimited play time)
  timeout = -1;

  pre[0]='\0';
  post[0]='\0';
  if(format!=0) *format='\0';

  filename2=strrchr(filename,'/');
  if(filename2==0) {
    filename2=filename;
  } else {
    filename2++;
  }

  c=strchr(filename2,'.');

  // return false if the name doesn't contain '.' char.
  if(c==0) return(rval);

  strncpy(pre,filename2,c-filename2);
  pre[c-filename2]='\0'; // make so that pre is null-terminated
  c=strrchr(filename2,'.');
  strcpy(post,c+1); // post is null-terminated

  if(strcasecmp(pre,"mp3")==0 || strcasecmp(post,"mp3")==0 ||
     strcasecmp(pre,"wav")==0 || strcasecmp(post,"wav")==0 ) {
    return(rval);
  }

  /* scans a our file and sets pre and post to right values if necessary
     (this is primarily useful when post/prefix doesn't tell us the
     exact format (like tfmx.*)) */
  if(filemagic_check) {
    uade_lock();
    scanfile(filename, pre, post);
    uade_unlock();
  }

  if(use_timeout==TRUE) {
    strcpy(a,timeout_val);
    c=strchr(a,':');
    if(c!=0) {
      *c=0;
      timeout=atoi(a)*60+atoi(c+1);
    } else {
      fprintf(stderr,"deliconfig: Illegal time format in timeout field!\n");
    }
  }

  if(optimize_filescan) {
    struct uade_format_list *node;


    if(uadeformats_is_cached) {

      node = uadeformats_head;

      if(node == 0) {
	fprintf(stderr,"This shouldn't happen: cache was reported to exist but it didn't!\nREPORT THIS BUG!! MEMORY HAS BEEN POSSIBLY LEAKED...\n");
	goto make_cache;
      }

      do {
	if(strncasecmp(node->ext,pre ,sizeof(node->ext))==0 ||
	   strncasecmp(node->ext,post,sizeof(node->ext))==0    )
	{
	  if(format) strcpy(format,node->player);
	  if(playername) strncpy(playername,node->playerfilepath,255);
	  return TRUE;
        }
	node = node->next;
      } while(node);

    } else {

    make_cache:

      while(uadeformats_head) {
	node = uadeformats_head->next;
	free(uadeformats_head);
	uadeformats_head = node;
      }
      /* uadeformats_head is zero */

      node = 0;

      /* find out file path containing uade formats information */
      resolve_path(formatsfilename,uade_formats_path);
      formatsfile=fopen(formatsfilename,"r");
      if(formatsfile == 0) {
	fprintf(stderr,"couldn't open config file (%s)!\n",formatsfilename);
	return FALSE;
      }
      for(;;) {
	x=fscanf(formatsfile,"%s",a);
	if(x==0 || x==EOF) break;
	
	// check if this is a comment line (this skips comment lines)
	if(a[0]=='#') {
	  while(a[0]!='\n') {
	    x=fscanf(formatsfile,"%c",&a[0]);
	    if(x==0 || x==EOF) break;
	  }
	  continue;
	}
	
	if(strcasecmp(a,"formats")==0) {
	  for(;;) {
	    x=fscanf(formatsfile,"%s",a);
	    if(x==0 || x==EOF) break;
	    if(strcasecmp("endformats",a)==0) break;
	    
	    // check if this is a comment line (this skips comment lines)
	    if(a[0]=='#') {
	      while(a[0]!='\n') {
		x=fscanf(formatsfile,"%c",&a[0]);
		if(x==0 || x==EOF) break;
	      }
	      continue;
	    }

	    x=fscanf(formatsfile,"%s",b);
	    if(x==0 || x==EOF) break;

	    if(node) {
	      node->next = malloc(sizeof(struct uade_format_list));
	      node = node->next;
	    } else {
	      uadeformats_head = malloc(sizeof(struct uade_format_list));
	      node = uadeformats_head;
	    }
	    node->next = 0;

	    strcpy(node->ext,a);
	    strcpy(node->player,b);
	    resolve_path(node->playerfilepath,player_dir);
	    if(node->playerfilepath[strlen(node->playerfilepath)-1]!='/') {
	      strcat(node->playerfilepath,"/");
	    }
	    strcat(node->playerfilepath,b);

	    if(strcasecmp(pre,a)==0 || strcasecmp(post,a)==0) {
	      if(format) strcpy(format,b);
	      if(playername) {
		resolve_path(playername,player_dir);
		if(playername[strlen(playername)-1]!='/') {
		  strcat(playername,"/");
		}
		strcat(playername,b);
	      }
	      rval=TRUE;
	    }
	  }
	  break;
	}
      }
      fclose(formatsfile);

      uadeformats_is_cached = 1;

    }

  } else {

    /* find out file path containing uade formats information */
    resolve_path(formatsfilename,uade_formats_path);
    
    formatsfile=fopen(formatsfilename,"r");
    if(formatsfile==0) {
      fprintf(stderr,"couldn't open config file (%s)!\n",formatsfilename);
      return FALSE;
    }
    
    for(;;) {
      x=fscanf(formatsfile,"%s",a);
      if(x==0 || x==EOF) break;
      
      // check if this is a comment line (this skips comment lines)
      if(a[0]=='#') {
	while(a[0]!='\n') {
	  x=fscanf(formatsfile,"%c",&a[0]);
	  if(x==0 || x==EOF) break;
	}
	continue;
      }
      
      if(strcasecmp(a,"formats")==0) {
	for(;;) {
	  x=fscanf(formatsfile,"%s",a);
	  if(x==0 || x==EOF) break;
	  if(strcasecmp("endformats",a)==0) break;

	  // check if this is a comment line (this skips comment lines)
	  if(a[0]=='#') {
	    while(a[0]!='\n') {
	      x=fscanf(formatsfile,"%c",&a[0]);
	      if(x==0 || x==EOF) break;
	    }
	    continue;
	  }
	  
	  x=fscanf(formatsfile,"%s",b);
	  if(x==0 || x==EOF) break;
	  
	  if(strcasecmp(pre,a)==0 || strcasecmp(post,a)==0) {
	    if(format) {
	      strcpy(format,b);
	      /* Find out deliplayer directory path */
	      if(playername) {
		resolve_path(playername,player_dir);
		if(playername[strlen(playername)-1]!='/') {
		  strcat(playername,"/");
		}
		strcat(playername,b);
	      }
	    }
	    rval=TRUE;
	    break;
	  }
	}
	break;
      }
    }
    fclose(formatsfile);
  }
  return(rval);
}


static void waitforuaetoreact(void) {
  while(uade_struct->touaemsgtype) {
    /* check if uade is alive */
    if(signal_slave(0) < 0) return;
    xmms_usleep(1000);
  }
}


/* this function definately needs cleanup. HINT: it copies song data from
   a circular shared memory buffer to XMMS sound buffers. The copied data
   is originated from uade executable (see src/sd-sound.h)
*/
static void *play_loop(void *arg) {
  const int samples = 512;
  const int blocksize = 4 * samples;
  char sndbuf[blocksize];
  short *sm;
  int l,r,m;
  int mixpar;
  int bytesfree;
  int zerocounter = 0;
  int zerolimit = uade_frequency*uade_bytespersample*uade_nchannels*30;
  int counter;
  int datainbuffer;

  while (playingbit) {

    bytesfree=deli_ip.output->buffer_free();

    if(bytesfree >= blocksize) {

      /* case 1: r=w: datainbuffer = 0      (consistent with case 2)
	 case 2: r<w: datainbuffer = w - r
	 case 3: r>w: datainbuffer = w + 32k - r
      */
      if(uade_struct->sbuf_readoffset <= uade_struct->sbuf_writeoffset) {
	/* case 1 and 2 */
	datainbuffer = uade_struct->sbuf_writeoffset - uade_struct->sbuf_readoffset;
      } else {
	/* case 3 */
	datainbuffer = uade_struct->sbuf_writeoffset + 32768 - uade_struct->sbuf_readoffset;
      }

      if(datainbuffer < blocksize) {
	xmms_usleep(10000);
	continue;
      }
      
      if((uade_struct->sbuf_readoffset + blocksize) > 32768) {
	int firstsize = 32768-uade_struct->sbuf_readoffset;
	memcpy(sndbuf,uade_struct->soundbuffer+uade_struct->sbuf_readoffset,firstsize);
	memcpy(sndbuf+firstsize,uade_struct->soundbuffer,blocksize-firstsize);
      } else {
	memcpy(sndbuf,uade_struct->soundbuffer+uade_struct->sbuf_readoffset,blocksize);
      }
      uade_struct->sbuf_readoffset = (uade_struct->sbuf_readoffset+blocksize)%32768;

      if(do_mixing) {
	sm = (short*) sndbuf;
	mixpar = (int) (mixing_parameter*(256.0f/2.0f));
	for(counter=0;counter<samples;counter++) {
	  l = (int) sm[0];
	  r = (int) sm[1];
	  m = (r-l) * mixpar;
	  sm[0] = (short) ( ((l<<8) + m) >> 8 );
	  sm[1] = (short) ( ((r<<8) - m) >> 8 );
	  sm += 2;
	}
      }

      deli_ip.add_vis_pcm(deli_ip.output->written_time(), uade_format, 2, blocksize, sndbuf);
      deli_ip.output->write_audio((void*) sndbuf, blocksize);

      // check if there's long silence in the tune => end song
      for(counter=0;counter<blocksize;counter++) {
	if(sndbuf[counter]!=0) {
	  zerocounter=0;
	  break;
	}
	zerocounter++;
      }
      if(zerocounter >= zerolimit) {
	zerocounter = 0;
	uade_struct->songendboolean = 1;
      }

    } else {
      xmms_usleep(10000);
    }
  }

  arg = 0; /* no warning */

  pthread_exit(0);
  return 0; /* no warning */
}


static void play_file(char *filename) {
  char format[256],text[256];
  int playtime;
  char *tempname;

  /* we do this before deli_file is set to non-zero for signal reasons */
  deli_file = 0;

  if(slavepid == 0) {
    int slept;
    forkexec();
    slept = 0;
    while(uade_struct->uade_inited_boolean == 0) {
      if(slept >= 5000000) {
	slavepid = -1;
	uade_struct->songendboolean = 1;
	fprintf(stderr,"uade: was not able to fork-exec uade executable, not playing\n");
	return;
      }
      xmms_usleep(100000);
      slept += 100000;
    }
  } else if(slavepid < 0) {
    uade_struct->songendboolean = 1;
    fprintf(stderr,"uade: was not able to fork-exec uade executable, not playing\n");
    return;
  }

  set_ntsc_pal(use_ntsc);
  set_song_end_possible(use_songend);

  if(playingbit) {
    fprintf(stderr,"uade: WARNING! A serious bug has been detected:\n");
    fprintf(stderr,"playingbit != 0 when play_file was called\n");
    fprintf(stderr,"SLEEP 1 second and hope it was just a bad dream...\n");
    xmms_usleep(1000000);
  }

  /* extract deli_file: the filename without path */
  tempname = strrchr(filename,'/');
  if(!tempname) {
    tempname = filename;
  } else {
    tempname++;
  }
  deli_file = g_malloc(strlen(tempname) + 1);
  strcpy(deli_file, tempname);

  if(deli_ip.output->open_audio(uade_format,uade_frequency,uade_nchannels)==0)
  {
    deli_file = 0;
    return;
  }

  /* invalidate format cache for each played song (i hope this doesn't leak
     memory) .. only useful if optimize_filescan is on */
  uadeformats_is_cached = 0;

  (void) check_my_file(filename,format,uade_struct->playername);

  /* check if this is a CUSTOM song */
  if(strcasecmp(format,"custom") == 0) {
    strcpy(uade_struct->playername,filename);
    uade_struct->modulename[0]='\0';
  } else {
    /* playername was set in check_my_file() */
    strcpy(uade_struct->modulename,filename);
  }

  /* re-read scorename */
  resolve_path(uade_struct->scorename,score_path);

  /* check song playtime from the database */
  if(db_loaded==0) setup_databases();
  db_get_content_hash(current_song_content_hash, filename);
  db_get_name_hash(current_song_name_hash, filename);
  database_playtime = db_check_song(songcontenttree, current_song_content_hash);
  if(database_playtime > 0) {
    playtime = database_playtime;
    if(db_check_song(songnametree, current_song_name_hash) == 0) {
      db_add_song_data(&songnametree, current_song_name_hash, playtime);
    }
  } else {
    database_playtime = 0;
    /* calculate song playtime according to timeout value */
    playtime = (timeout==-1) ? 0 : (1000*timeout);
  }
  cumulative_playtime = 0;

  /* Tell song len to XMMS, set XMMS window scroller text */
  strcpy(text,"");
  strcat(text,tempname);
  strcat(text," [");
  strcat(text,format);
  strcat(text,"]");
  deli_ip.set_info_text(text);
  deli_ip.set_info(text, playtime, 8*4*uade_frequency, uade_frequency, uade_nchannels);

  uade_struct->force_by_default = force_by_default;
  uade_struct->setsubsongboolean = 0;
  uade_struct->subsong = 0;
  uade_struct->dontwritebit = 0;
  uade_struct->songendboolean = 0;

  uade_struct->plugin_pause_boolean = 0;
  uade_struct->sbuf_writeoffset = 0;
  uade_struct->sbuf_readoffset = 0;

  playingbit = 1;

  /* this syncs playername, modulename and scorename with uade internal
     structures */
  uade_struct->loadnewsongboolean = 1;
  uade_struct->touaemsgtype=UADE_PLAYERNAME;
  if(signal_slave(0) >= 0) signal_slave(SIGHUP);

  pthread_create(&decode_thread, 0, play_loop, 0);
}


static void stop(void) {
  /* invalidate uadeformats cache.. only useful if optimize_filescan is on */
  uadeformats_is_cached = 0;

  if (deli_file) {

    /* this will make play_loop thread terminate itself in time */
    playingbit = 0;

    /* forbid uade to write into shared memory */
    uade_struct->dontwritebit = 1;

    uade_struct->plugin_pause_boolean = 0;

    uade_struct->loadnewsongboolean = 0;

    /* check if uade is alive */
    if(signal_slave(0) >= 0) {
      // fprintf(stderr,"uade/deli.c: stop: SENDING REBOOT SIGNAL TO UAE!\n");
      uade_struct->touaemsgtype=UADE_REBOOT;
      signal_slave(SIGHUP);
      waitforuaetoreact();
    }

    pthread_join(decode_thread, 0);

    deli_ip.output->close_audio();

    free(deli_file); deli_file = 0;

    current_song_name_hash[0]=0;

    /* at last removing windows belonging to us */
    deli_close_win();
  }
}


static void deli_pause(short p) {
  uade_struct->plugin_pause_boolean = (p==0) ? 0 : 1;
  deli_ip.output->pause(p);
}


/* if seek_next or seek_previous returns -1 then subsong is not changed */
int seek_next() {
  int newsubsong;
  if(uade_struct->plugin_pause_boolean == 0) {
    newsubsong = get_curr_subsong();
    newsubsong++;
    return newsubsong;    
  } else  {
    return -1;
  }
}


int seek_previous() {
  int newsubsong;
  if(uade_struct->plugin_pause_boolean == 0) {
    newsubsong = get_curr_subsong();
    if (newsubsong > uade_struct->minsubsong) {
      newsubsong--;
      return newsubsong;
    }
  }
  return -1;
}


void seek (int newsubsong) {
  fprintf(stderr,"uade/deli.c seek(): seeking to subsong %d\n",newsubsong);
  uade_struct->touaemsgtype=UADE_SETSUBSONG;
  uade_struct->setsubsongboolean = 1;
  uade_struct->subsong = newsubsong;
  /* we would like to change subsong when there are sound data in buffer
     left but score has reported song end */
  uade_struct->songendboolean = 0;
  /* if subsong is changed manually cumulative_playtime is rendered invalid */
  cumulative_playtime = -1;
  signal_slave(SIGHUP);
  waitforuaetoreact();
  deli_ip.output->flush(0);
}


static int get_time(void) {
  int currenttime = deli_ip.output->output_time();
  static int endinprogress = 0;
  static int endtime;

  /* First time songend is noticed we do following:
     1. mark that end is in progress (static int endinprogress = 1)
     2. save the output plugin written time for the purpose of determining
        later when to stop playing.. call this saved value endtime.
     3. check if output time is at least endtime
        true  => end song by setting endinprogess=0 and returning -1
        false => return playtime normally

     Second time songend is noticed we only execute step 3.

     If songend is not detected, we set endinprogess to 0 to be certain
     that when song end is noticed the first time previous procedure
     goes smoothly..
  */

  if(uade_struct->songendboolean) {
    /* if uade executable is not functional this will cause song end */
    if(deli_file == 0) {
      return -1;
    }
    if(endinprogress == 0) {
      endtime = deli_ip.output->written_time();
      endinprogress = 1;
      /* if a subsong was changed manually, cumulative_playtime became -1
	 and thus invalid */
      if(cumulative_playtime >= 0) {
	/* add cumulative_playtime to find out total duration of all subsongs
	 */
	fprintf(stderr,"updating cumulative_playtime: %d -> %d\n",cumulative_playtime,cumulative_playtime+endtime);
	cumulative_playtime += endtime;
      } else {
	fprintf(stderr,"invalid cumulative_playtime => not adding\n");
      }
      fprintf(stderr,"first notice: output_time = %d written_time = %d\n",deli_ip.output->output_time(),endtime);
    }
    if(endinprogress) {
      // fprintf(stderr,"second notice: output_time = %d written_time = %d\n",deli_ip.output->output_time(),endtime);
      if(endtime <= deli_ip.output->output_time()) {
	endinprogress = 0;
	if(use_sequential_subsongs==TRUE && get_max_subsong()!=0) {
	  int cur = get_curr_subsong();
	  if(cur < get_max_subsong()) {
	    int cumulative_playtime_backup = cumulative_playtime;
	    seek(cur+1);
	    cumulative_playtime = cumulative_playtime_backup;
	    return 0;
	  }
	}
	if(cumulative_playtime>=0) {
	  db_add_song_data(&songcontenttree, current_song_content_hash, cumulative_playtime);
	  db_add_song_data(&songnametree, current_song_name_hash, cumulative_playtime);
	} else {
	  fprintf(stderr,"uade/seek: invalid cumulative playtime => not adding song\n");
	}
	return -1;
      }
    }
  } else {
    endinprogress = 0;
    if (timeout > 0) {
      int outputtime = deli_ip.output->output_time();
      if((outputtime/1000) >= timeout) {
	if(database_playtime <= outputtime) return -1;
      }
    }
  }
  return currenttime;
}


int get_curr_subsong(void) {
  /*  if (uade_struct->subsong < uade_struct->minsubsong){
    return uade_struct->minsubsong;
  } else {
    return uade_struct->subsong;
  }
  */
  return uade_struct->subsong;
}


int get_min_subsong(void) {
  return uade_struct->minsubsong;
}


int get_max_subsong(void) {
  return uade_struct->maxsubsong;
}


char* get_modulename(void) {
  if(uade_struct->deli_modulename[0]!='\0') {
    return uade_struct->deli_modulename;
  } else if (deli_file!=0) {
    return deli_file;
  } else {
    return "<no name>"; //filename;
  }
}


char* get_playername(void) {
  if(uade_struct->deli_playername[0]!='\0') {
    return uade_struct->deli_playername;
    } else {
    return "Custom"; // Assumptions are always bad, I know ;)
  }
}


static void get_song_info(char *filename, char **title, int *length) {
  char *name,*temp;
  char md5namehash[33];
  int songlength;

  if(db_loaded == 0) setup_databases();

  /* no length for song defined by default */
  (*length) = -1;

  /* check name database for song length */
  if(db_get_name_hash(md5namehash, filename)) {
    songlength = db_check_song(songnametree,md5namehash);
    if(songlength) {
      *length = songlength;
    }
  }
  /*  if(db_get_content_hash(md5namehash,filename)) {
      songlength = db_check_song(songcontenttree,md5namehash);
      if(songlength) {
      *length = songlength;
      }
      }
  */
  temp = strrchr(filename,(int) '/');
  if(temp==0) {
    temp=filename;
  } else {
    temp++;
  }
  name = g_malloc(strlen(temp)+1);
  strcpy(name,temp);

  (*title) = name;
}


static void my_fileinfo(char *filename) {
  strcpy(fileinfo_playerauthor,"");
  strcpy(fileinfo_playerversion,"");
  strcpy(fileinfo_songend,"");
  strcpy(fileinfo_modulecomment,"Played with UADE");
  deli_fileinfo();
  filename = 0; /* no warning */
}


void set_ntsc_pal(gboolean new_ntsc_bit) {
  use_ntsc = new_ntsc_bit;
  if (uade_struct) {
    uade_struct->ntscbit = use_ntsc==FALSE ? 0:1;
    if (deli_file) {
      /* check if uade is alive */
      if(signal_slave(0) >= 0) {
	// fprintf(stderr,"sending ntscbit = %d\n",uade_struct->ntscbit);
	uade_struct->touaemsgtype=UADE_NTSC;
	signal_slave(SIGHUP);
	waitforuaetoreact();
      }
    }
  }
}


void set_song_end_possible(gboolean use_songend) {
  if(uade_struct) {
    uade_struct->songendpossible = use_songend ? -1:0;
    if(deli_file) {
      if(signal_slave(0) >= 0) {
	uade_struct->touaemsgtype=UADE_SONGEND;
	signal_slave(SIGHUP);
	waitforuaetoreact();
      }
    }
  }
}


static int db_read_data(tnode **root, const char *filename) {
  FILE *dbFile;
  char md5[34];
  int x;
  char lbuf[16];
  int length;
  dbFile = fopen(filename,"r");
  if(dbFile == 0) {
    return 0;
  }
  while(1) {
    x = fscanf(dbFile,"%32s %d\n",md5,(int*) lbuf);
    if(x==2) {
      length = *((int*)lbuf);
      // fprintf(stderr,"uade/db_read_data: add %s %d\n",md5,length);
      if(strlen(md5)==32) {
	db_add_song_data(root, md5, length);
      } else {
	fprintf(stderr,"uade/db_read_data: illegal data (%s %d)\n",md5,length);
      }
    } else {
      break;
    }
  }
  fclose(dbFile); dbFile = 0;
  return 1;
}


static void db_write_data(const char *filename, tnode *root) {
  FILE *dbFile;
  if(root) {
    dbFile = fopen(filename,"w");
    if(dbFile == 0) {
      fprintf(stderr,"uade: couldn't open database %s for writing\n",filename);
      return;
    }
    btree_traverse(root,uadedbwriteroutine,(void*) dbFile);
    fclose(dbFile); dbFile = 0;
  }
}


static int uadedbwriteroutine(tnode *node, void *arg) {
  int length;
  length = *((int*) node->data);
  // fprintf(stderr,"uade/db_write_data: %s %d\n",node->key.key, length);
  if(strlen(node->key.key)==32) {
    fprintf((FILE*) arg,"%s %d\n",node->key.key, length);
  } else {
    fprintf(stderr,"uade/db_write_data: illegal data (%s %d)\n",node->key.key,length);
  }
  return 1;
}


static int db_check_song(tnode *root, const char *hash) {
  tnode *node;
  tnodekey key;
  strncpy(key.key,hash,32);
  key.key[32]=0;
  /* no problem if root = 0 */
  node = btree_addnode(root, 0, &key, 0);
  if(node) {
    int *dblength = (int*) node->data;
    if(strlen(node->key.key)==32) {
      // fprintf(stderr,"db_check_song: playtime = %d\n",*dblength);
      return *dblength;
    } else {
      fprintf(stderr,"uade/db_check_song: BUG %s %d\n",node->key.key,*dblength);
    }
  }
  return 0;
}


static int db_get_content_hash(char *hash, const char *filename) {
  if(hash) {
    int retval = filechecksum(hash,filename);
    hash[32]=0;
    return retval;
  }
  return 0;
}


static int db_get_name_hash(char *hash, const char *filename) {
  char md5[33];
  int filesize;
  struct stat Stat;
  if(hash) {
    hash[0]=0;
    if(stat(filename,&Stat)==0) {
      filesize = Stat.st_size;
      strchecksum(md5,filename);
      sprintf(hash,"%.8x",filesize);
      strncat(hash,md5,24);
      return 1;
    }
  }
  return 0;
}


static void db_add_song_data(tnode **root, char *hash, int length) {
  tnode *node;
  tnodekey key;
  if(hash) {
    if(strlen(hash)!=32) {
      fprintf(stderr,"uade/db_add_song_data: possible BUG:\n");
      fprintf(stderr,"\t%s %s %d\n",deli_file?deli_file:"null",hash,length);
      return;
    }
    strncpy(key.key,hash,32);
    key.key[32]=0;
    node = btree_addnode(*root, 0, &key, 0);
    if(node) {
      int *dblength = (int*) node->data;
      *dblength = length;
      // fprintf(stderr,"setting new length %d for existing song (hash %s)\n",length,key.key);
    } else {
      int *newdata = malloc(sizeof(int));
      *newdata = length;
      // fprintf(stderr,"adding new song (hash %s) with length %d\n",key.key,length);
      if(*root) {
	btree_addnode(*root, (void*) newdata, &key, 1);
      } else {
	*root = btree_createtree(newdata,&key);
      }
    }
  }
}
